#version 330 core

in vec3 vNormal;
in vec2 vUV;
in vec3 fragPos;
in vec4 fragPosLightSpace;

out vec4 fragColor;

uniform sampler2D shadowMap;

vec3 lightDir = vec3(50.0f, 50.0f, 0.0f);
vec3 lightColor;

// void main() {
//     vec3 texSamp = texture(tex, vUV).rgb;
//     lightColor = texSamp;
//     vec3 norm = normalize(vNormal);
//     vec3 lightDir = normalize(lightDir);
//     float diff = max(dot(norm, lightDir), 0.0);
//     vec3 diffuse = diff * lightColor;
//     fragColor = vec4(diffuse, 1.0f) + vec4((texSamp * 0.8f) * vec3(0.445f, 0.717f, 0.867f), 1.0f);
// }

float shadowCalculation(vec4 fragPosLightSpace) {
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // check whether current frag pos is in shadow
    float bias = 0.001;
    float shadow = 0.0;
    vec2 texelSize = 0.9453 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;
    
    // keep the shadow at 0.0 when outside the far_plane region of the light's frustum.
    if(projCoords.z > 1.0)
        shadow = 0.0;

    return shadow;
}

void main() {
    float shadow = shadowCalculation(fragPosLightSpace);
    vec3 texSamp = vec3(1.0f);
    lightColor = texSamp;
    vec3 norm = normalize(vNormal);
    vec3 lightDir = normalize(lightDir);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = diff * lightColor;
    fragColor = vec4(diffuse * (0.7 - shadow), 1.0f) + vec4((texSamp * 0.8f) * vec3(0.445f, 0.717f, 0.867f), 1.0f);
}